from typing import Type
from dataclasses import dataclass
from .definition import JsonRPCHandler
from marshmallow import Schema
from .exceptions import JsonRpcException, OPERATION_NOT_FOUND_ERROR

REGISTRY_SETTINGS_KEY = 'operations_registry'


@dataclass
class Operation:
    name: str
    handler: Type[JsonRPCHandler]
    request_schema: Schema = None
    response_schema: Schema = None


class OperationRegistry:
    storage = {}

    def handler(self, operation_name: str, request_schema: Schema = None, response_schema: Schema = None):
        def wrapper(cls):
            cls.__operation__ = operation_name
            self.add(Operation(
                name=operation_name,
                request_schema=request_schema,
                response_schema=response_schema,
                handler=cls
            ))
            return cls

        return wrapper

    def add(self, operation: Operation):
        self.storage[operation.name] = operation

    def get(self, name: str) -> Operation:
        result = self.storage.get(name)
        if not result:
            raise JsonRpcException(
                code=OPERATION_NOT_FOUND_ERROR,
                message="Operation not found"
            )
        return result

    def all(self):
        return list(self.storage.values())
